# Copyright (C) 2022 Cochise Ruhulessin
#
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
from typing import TypeAlias

from .deletedevent import DeleteEvent
from .event import Event
from .ping import Ping
from .orderevent import OrderEvent
from .productevent import ProductEvent


__all__: list[str] = [
    'DeleteEvent',
    'Event',
    'EventType',
    'OrderEvent',
    'ProductEvent',
    'Ping'
]

EventType: TypeAlias = (
    OrderEvent | ProductEvent | DeleteEvent | Ping
)